/* ============================================================================
 * Copyright (c) 2011 Michael A. Jackson (BlueQuartz Software)
 * Copyright (c) 2011 Dr. Michael A. Groeber (US Air Force Research Laboratories)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this
 * list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *
 * Neither the name of Michael A. Groeber, Michael A. Jackson, the US Air Force,
 * BlueQuartz Software nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  This code was written under United States Air Force Contract number
 *                           FA8650-07-D-5800
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

#ifndef H5VOXELFILEREADER_H_
#define H5VOXELFileREADER_H_

#include <string>
#include <vector>

#include "DREAM3DLib/DREAM3DLib.h"
#include "DREAM3DLib/Common/DREAM3DSetGetMacros.h"
#include "DREAM3DLib/Common/DataArray.hpp"
#include "DREAM3DLib/Common/FileReader.h"

#include "DREAM3DLib/HDF5/H5VoxelReader.h"


/**
 * @class H5VoxelFileReader H5VoxelFileReader.h DREAM3DLib/HDF5/H5VoxelFileReader.h
 * @brief This class is meant to read the grain ids out of an h5voxel file and nothing
 * else.
 * @author Michael A. Jackson for BlueQuartz Software
 * @date Dec 21, 2011
 * @version 1.0
 */
 class H5VoxelFileReader : public FileReader
  {
    public:
      DREAM3D_SHARED_POINTERS(H5VoxelFileReader)
      DREAM3D_STATIC_NEW_MACRO(H5VoxelFileReader)
      DREAM3D_TYPE_MACRO_SUPER(H5VoxelFileReader, FileReader)


      virtual ~H5VoxelFileReader();

    DREAM3D_INSTANCE_STRING_PROPERTY(InputFile)

	//------ Created Cell Data
    DREAM3D_INSTANCE_STRING_PROPERTY(GrainIdsArrayName)
    DREAM3D_INSTANCE_STRING_PROPERTY(CellPhasesArrayName)
    DREAM3D_INSTANCE_STRING_PROPERTY(CellEulerAnglesArrayName)
    //------ Created Ensemble Data
    DREAM3D_INSTANCE_STRING_PROPERTY(CrystalStructuresArrayName)
    DREAM3D_INSTANCE_STRING_PROPERTY(PhaseTypesArrayName)

    virtual const std::string getGroupName() { return DREAM3D::FilterGroups::IOFilters; }
    virtual const std::string getHumanLabel() { return "Read Legacy DREAM.3D H5Voxel File"; }

	  virtual void setupFilterParameters();
	  virtual void writeFilterParameters(AbstractFilterParametersWriter* writer);

	  virtual void preflight();
	  virtual void execute();

    protected:
      H5VoxelFileReader();

    private:
      int32_t* m_GrainIds;
      int32_t* m_CellPhases;
	  float* m_CellEulerAngles;

	  unsigned int* m_CrystalStructures;
	  unsigned int* m_PhaseTypes;

    void dataCheck(bool preflight, size_t voxels, size_t fields, size_t ensembles);

	  H5VoxelFileReader(const H5VoxelFileReader&); // Copy Constructor Not Implemented
      void operator=(const H5VoxelFileReader&); // Operator '=' Not Implemented

  };


#endif /* H5VoxelFileReader_H_ */
