
/* ============================================================================
* Copyright (c) 2012 Dr. Anthony Rollet (Carnegie-Mellon University)
* Copyright (c) 2012 Dr. Sukbin Lee (Carnegie-Mellon University)
* Copyright (c) 2012 Dr. Jason Gruber (Carnegie-Mellon University)
* Copyright (c) 2012 Dr. Stephen Sintay (Carnegie-Mellon University)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this
* list of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or
* other materials provided with the distribution.
*
* Neither the name of Anthony Rollet, Sukbin Lee, Jason Gruber, Stephen Sintay
* nor the names of its contributors may be used to endorse
* or promote products derived from this software without specific prior written
* permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
* FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
// smooth.cpp

//  g++ -g -O3 -o smooth smooth-grain3d-ADR.cpp

// Written by Jason Gruber jgruber@andrew.cmu.edu
//   many mods by Steve Sintay, ADR

//  added an option to turn off edge-of-model constraints
//  ADR, Nov 09; found another place to turn off dc/dt, Dec 09
//  ADR, Mar 10; added option to constrain quad points
//  ADR, May 10; added node info to output to track
//               problems with surface pull-in along GBs
//               changed the way in which constraints are added, and used
//  ADR, June 10; added edge info to compute TJ smoothing forces
//      created a new class for edges, but do not need it, in fact
//      created a function of line curvature for computing forces
//      also added option to restart from an INP
//       Feb 11; debugging surface node constraint

//  look for mobility values etc, and perhaps tweak them

#ifndef _MFE_NODE_H_
#define _MFE_NODE_H_




class node
{
    friend int& id(node&);
    friend int& type(node&);
    friend int& triplenn1(node&);
    friend int& triplenn2(node&);
  public:
    double& operator[](int i)
    {
      return x[i];
    }
    double operator[](int i) const
    {
      return x[i];
    }
    void print(std::ostream &out)
    {
      out << "id: " << id << std::endl;
      out << "type: " << type << std::endl;
      out << "triplenn1: " << triplenn1 << std::endl;
      out << "triplenn2: " << triplenn2 << std::endl;
      out << "x: " << x[0] << ", " << x[1] << ", " << x[2] << std::endl;
    }
  private:
    double x[3];
    int id;
    int type;
    int triplenn1; //  for storing IDs or NN nodes on triple lines
    int triplenn2;
};

inline int& id(node& n)
{
  return n.id;
}
inline int& type(node& n)
{
  return n.type;
}
inline int& triplenn1(node& n)
{
  return n.triplenn1;
}
inline int& triplenn2(node& n)
{
  return n.triplenn2;
}

/*
 double node::TJlength() {
 return distance( *id , *triplenn2 ) + distance(  *triplenn1 , *id ) ;
 }
 */

double distance(const node& n0, const node& n1)
{
  double a = n0[0] - n1[0];
  double b = n0[1] - n1[1];
  double c = n0[2] - n1[2];
  return sqrt(a * a + b * b + c * c);
}

#endif /* _MFE_NODE_H_ */
