
/* ============================================================================
 * Copyright (c) 2012 Dr. Anthony Rollet (Carnegie-Mellon University)
 * Copyright (c) 2012 Dr. Sukbin Lee (Carnegie-Mellon University)
 * Copyright (c) 2012 Dr. Jason Gruber (Carnegie-Mellon University)
 * Copyright (c) 2012 Dr. Stephen Sintay (Carnegie-Mellon University)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this
 * list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *
 * Neither the name of Anthony Rollet, Sukbin Lee, Jason Gruber, Stephen Sintay
 * nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
// smooth.cpp

//  g++ -g -O3 -o smooth smooth-grain3d-ADR.cpp

// Written by Jason Gruber jgruber@andrew.cmu.edu
//   many mods by Steve Sintay, ADR

//  added an option to turn off edge-of-model constraints
//  ADR, Nov 09; found another place to turn off dc/dt, Dec 09
//  ADR, Mar 10; added option to constrain quad points
//  ADR, May 10; added node info to output to track
//               problems with surface pull-in along GBs
//               changed the way in which constraints are added, and used
//  ADR, June 10; added edge info to compute TJ smoothing forces
//      created a new class for edges, but do not need it, in fact
//      created a function of line curvature for computing forces
//      also added option to restart from an INP
//       Feb 11; debugging surface node constraint

//  look for mobility values etc, and perhaps tweak them



#ifndef MFE_SMOOTH_H_
#define MFE_SMOOTH_H_

#include <string>

class MFESmooth
{
  public:
    MFESmooth();
    virtual ~MFESmooth();

    /**
     * @brief Cancels the smoothing algorithm
     */
    void cancel();

    /**
     * @brief Print messages to standard output
     * @param v True to print messages
     */
    void printMessages(bool v);

    /**
     * @brief Performs a Moving Finite Element Surface Mesh smoothing algorithm
     * @param nodesFile Input Nodes File
     * @param triangleFile Input Triangles File
     * @param edgesFile Input Edges File
     * @param steps Number of smoothing steps
     * @param constrain Constrain all nodes
     * @param constrainSurface Constrain Surface Nodes
     * @param constrainQuads Constrain Quad Points
     * @param smoothTripleLines Smooth Triple Lines
     * @param restartFile Restart File
     * @param outputNodesFile Output Nodes File
     * @return Zero on success, -1 on Error
     */
    int execute(const std::string &nodesFile,
                  const std::string &triangleFile,
                  const std::string &edgesFile,
                  int steps,
                  int constrain,
                  int constrainSurface,
                  int constrainQuads,
                  int smoothTripleLines,
                  const std::string &restartFile,
                  const std::string &outputNodesFile);

  private:
    bool m_Cancel;
    bool m_PrintMessages;

    MFESmooth(const MFESmooth&); // Copy Constructor Not Implemented
    void operator=(const MFESmooth&); // Operator '=' Not Implemented
};




#endif /* MFE_SMOOTH_H_ */
